;; Calculate path lengths according to number of edges.
;; For use with path.lisp (q.v.)
;; Invoke  (plength) after running (path --)
;; Uses `cnd', but leaves it = nil!!
;; For every receiver leaves
;;    mymx = maximum no. of edges in any optimum path to recvr
;;    mymn = minimum no. of edges in any optimum path to recvr

(in-package '*lisp)

(*proclaim '(type (signed-pvar 32) cld mymx mymn))
(*defvar clng (!! 0))      ;; Length of path according to edge count
(*defvar mymx (!! INF))    ;; Max length to this receiver
(*defvar mymn (!! INF))    ;; Min length to this receiver

(defun plength ()
  ;; Initialize for edge counts
  (*set mymx (!! INF))  (*set mymn (!! INF))  (*set cnd nil!!)
  (*when (zerop!! ptd)  ;; Source nodes
    (*set mymx (!! 0))  (*set mymn (!! 0)) (*set cnd t!!))
  ;; Loop to propagate minimum counts
  (do ((more t) (k 1 (incf k)))
      ((not more) )
    (*when cnd        ;; Nodes for which min  = k - 1
      (*set cnd nil!!)
      (*when (plusp!! lng)
        (*pset :no-collisions (!! k) mymn to))
    )  ;; end *when cnd
    (*set clng (reduce-and-spread!! mymn 'min!! 1))
    (*set cnd (=!! clng (!! k)))
    (setq more (*or cnd))
  )  ;; end do for min path lengths
  ;; Now cnd = nil!!
  (*set cnd (zerop!! ptd))  ;; Source nodes
  ;; Loop to calculate maximum path lengths
  (do ((more t) (k 1 (incf k)))
      ((not more))
    (*when cnd        ;; Nodes for which max = k-1
      (*set cnd nil!!)
      (*when (plusp!! lng)
        (*pset :no-collisions (!! k) mymx to))
    )  ;; end *when cnd
    (*set clng
          (reduce-and-spread!! (if!! shrt mymx (!! 0)) 'max!! 1))
    (*set cnd (=!! clng (!! k)))
    (setq more (*or cnd))
  )  ;; end do for max path lengths
  (*when shrt
    (list 'MAXMAX (*max mymx)
          'MAXMIN (*max mymn)
          'MAX-MIN (*max (-!! mymx mymn))))
) ;; end defun plength
